@@ -10,7 +10,7 @@
 #include <cstdio>    // for FILE, fread
 #include <cstring>   // for memcpy
 #include <cmath>     // for isnan
-#include <algorithm> // for sort
+#include <algorithm> // for sort, any_of
 #include <array>
 #include <string>
 #include <vector>
@@ -19,13 +19,31 @@
 #include "iterator.hpp"  // for StrideIter
 #include "fail.hpp"      // for fail
 #include "fileutil.hpp"  // for file_open, is_little_endian, fileptr_t, ...
+#include "math.hpp"      // for rad, Mat33
 #include "symmetry.hpp"  // for find_spacegroup_by_name, SpaceGroup
 #include "unitcell.hpp"  // for UnitCell
 #include "util.hpp"      // for ialpha4_id, rtrim_str, ialpha3_id, ...
-#include "reciproc.hpp"  // for calculate_min_max_disregarding_nans
 
 namespace gemmi {
 
+template <typename T, typename FP=typename std::iterator_traits<T>::value_type>
+std::array<FP,2> calculate_min_max_disregarding_nans(T begin, T end) {
+  std::array<FP,2> minmax = {{NAN, NAN}};
+  T i = begin;
+  while (i != end && std::isnan(*i))
+    ++i;
+  if (i != end) {
+    minmax[0] = minmax[1] = *i;
+    while (++i != end) {
+      if (*i < minmax[0])
+        minmax[0] = *i;
+      else if (*i > minmax[1])
+        minmax[1] = *i;
+    }
+  }
+  return minmax;
+}
+
 // Unmerged MTZ files always store in-asu hkl indices and symmetry operation
 // encoded in the M/ISYM column. Here is a helper for writing such files.
 struct UnmergedHklMover {
@@ -36,21 +54,9 @@ struct UnmergedHklMover {
 
   // Modifies hkl and returns ISYM value for M/ISYM
   int move_to_asu(std::array<int, 3>& hkl) {
-    int isym = 1;
-    for (Op op : group_ops_) {
-      Miller new_hkl = op.apply_to_hkl(hkl);
-      if (asu_.is_in(new_hkl)) {
-        hkl = new_hkl;
-        return isym;
-      }
-      Miller negated_new_hkl{{-new_hkl[0], -new_hkl[1], -new_hkl[2]}};
-      if (asu_.is_in(negated_new_hkl)) {
-        hkl = negated_new_hkl;
-        return isym + 1;
-      }
-      isym += 2;
-    }
-    return 0;
+    std::pair<Miller, int> hkl_isym = asu_.to_asu(hkl, group_ops_);
+    hkl = hkl_isym.first;
+    return hkl_isym.second;
   }
 
 private:
@@ -289,6 +295,8 @@ struct Mtz {
     return data.size() == columns.size() * nreflections;
   }
 
+  bool is_merged() const { return batches.empty(); }
+
   void extend_min_max_1_d2(const UnitCell& uc, double& min, double& max) const {
     for (size_t i = 0; i < data.size(); i += columns.size()) {
       double res = uc.calculate_1_d2_double(data[i+0], data[i+1], data[i+2]);
@@ -742,11 +750,11 @@ struct Mtz {
     }
   }
 
-  void set_data(const float* new_data, int n) {
-    int ncols = (int) columns.size();
+  void set_data(const float* new_data, size_t n) {
+    size_t ncols = columns.size();
     if (n % ncols != 0)
       fail("Mtz.set_data(): expected " + std::to_string(ncols) + " columns.");
-    nreflections = n / ncols;
+    nreflections = int(n / ncols);
     data.assign(new_data, new_data + n);
   }
 
@@ -775,10 +783,17 @@ struct MtzDataProxy {
   const Mtz& mtz_;
   size_t stride() const { return mtz_.columns.size(); }
   size_t size() const { return mtz_.data.size(); }
+  using num_type = float;
   float get_num(size_t n) const { return mtz_.data[n]; }
   const UnitCell& unit_cell() const { return mtz_.cell; }
   const SpaceGroup* spacegroup() const { return mtz_.spacegroup; }
   Miller get_hkl(size_t offset) const { return mtz_.get_hkl(offset); }
+
+  size_t column_index(const std::string& label) const {
+    if (const Mtz::Column* col = mtz_.column_with_label(label))
+      return col->idx;
+    fail("MTZ file has no column with label: " + label);
+  }
 };
 
 // Like above, but here the data is stored outside of the Mtz class
@@ -795,6 +810,8 @@ struct MtzExternalDataProxy : MtzDataProxy {
   }
 };
 
+inline MtzDataProxy data_proxy(const Mtz& mtz) { return {mtz}; }
+
 } // namespace gemmi
 
 #ifdef GEMMI_WRITE_IMPLEMENTATION
@@ -805,7 +822,8 @@ namespace gemmi {
 
 #define WRITE(...) do { \
     int len = gf_snprintf(buf, 81, __VA_ARGS__); \
-    std::memset(buf + len, ' ', 80 - len); \
+    if (len < 80) \
+      std::memset(buf + len, ' ', 80 - len); \
     if (std::fwrite(buf, 80, 1, stream) != 1) \
       fail("Writing MTZ file failed"); \
   } while(0)
@@ -839,7 +857,7 @@ void Mtz::write_to_stream(std::FILE* stream) const {
         ops.order(),               // number of symmetry operations
         (int) ops.sym_ops.size(),  // number of primitive operations
         lat_type,                  // lattice type
-        spacegroup->ccp4,          // space group number 
+        spacegroup->ccp4,          // space group number
         20 - (int) std::strlen(spacegroup->hm), "",
         lat_type,                  // space group name (first letter)
         spacegroup->hm + 1,        // space group name (the rest)
@@ -856,10 +874,17 @@ void Mtz::write_to_stream(std::FILE* stream) const {
     WRITE("VALM NAN");
   else
     WRITE("VALM %f", valm);
+  auto format17 = [](float f) {
+    char buffer[18];
+    int len = gf_snprintf(buffer, 18, "%.9f", f);
+    return std::string(buffer, len > 0 ? std::min(len, 17) : 0);
+  };
   for (const Column& col : columns) {
     auto minmax = calculate_min_max_disregarding_nans(col.begin(), col.end());
-    WRITE("COLUMN %-30s %c %17.9f %17.9f %4d",
-          col.label.c_str(), col.type, minmax[0], minmax[1], col.dataset_id);
+    WRITE("COLUMN %-30s %c %17s %17s %4d",
+          col.label.c_str(), col.type,
+          format17(minmax[0]).c_str(), format17(minmax[1]).c_str(),
+          col.dataset_id);
     if (!col.source.empty())
       WRITE("COLSRC %-30s %-36s  %4d",
             col.label.c_str(), col.source.c_str(), col.dataset_id);
