@@ -88,6 +88,10 @@ inline std::string read_string(const char* p, int field_length) {
 inline bool is_record_type(const char* s, const char* record) {
   return ialpha4_id(s) == ialpha4_id(record);
 }
+// for record "TER": "TER ", TER\n, TER\r, TER\t match, TERE, TER1 don't
+inline bool is_record_type3(const char* s, const char* record) {
+  return (ialpha4_id(s) & ~0xf) == ialpha4_id(record);
+}
 
 // The standard charge format is 2+, but some files have +2.
 inline signed char read_charge(char digit, char sign) {
@@ -281,7 +285,7 @@ inline bool same_str(const std::string& s, const char (&literal)[N]) {
 
 template<typename Input>
 Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
-                                   int max_line_length) {
+                                   const PdbReadOptions& options) {
   using namespace pdb_impl;
   int line_num = 0;
   auto wrong = [&line_num](const std::string& msg) {
@@ -296,6 +300,7 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
   Chain *chain = nullptr;
   Residue *resi = nullptr;
   char line[122] = {0};
+  int max_line_length = options.max_line_length;
   if (max_line_length <= 0 || max_line_length > 120)
     max_line_length = 120;
   bool after_ter = false;
@@ -350,9 +355,14 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
       atom.pos.z = read_double(line+46, 8);
       atom.occ = (float) read_double(line+54, 6);
       atom.b_iso = (float) read_double(line+60, 6);
-      bool has_elem = len > 76 && (std::isalpha(line[76]) ||
-                                   std::isalpha(line[77]));
-      atom.element = Element(line + (has_elem ? 76 : 12));
+      if (len > 76 && (std::isalpha(line[76]) || std::isalpha(line[77])))
+        atom.element = Element(line + 76);
+      // Atom names HXXX are ambiguous, but Hg, He, Hf, Ho and Hs (almost)
+      // never have 4-character names, so H is assumed.
+      else if (alpha_up(line[12]) == 'H' && line[15] != ' ')
+        atom.element = El::H;
+      else
+        atom.element = Element(line + 12);
       atom.charge = (len > 78 ? read_charge(line[78], line[79]) : 0);
       resi->atoms.emplace_back(atom);
 
@@ -372,7 +382,11 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
       atom.aniso.u23 = read_int(line+63, 7) * 1e-4f;
 
     } else if (is_record_type(line, "REMARK")) {
-      st.raw_remarks.push_back(line);
+      if (line[len-1] == '\n')
+        --len;
+      if (line[len-1] == '\r')
+        --len;
+      st.raw_remarks.emplace_back(line, line+len);
       if (len <= 11)
         continue;
       int num = read_int(line + 7, 3);
@@ -398,7 +412,7 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
         if (starts_with(line+11, "  BIOMT")) {
           if (read_matrix(matrix, line+13, len-13) == 3)
             if (!assembly.generators.empty()) {
-              auto& opers = assembly.generators.back().opers;
+              auto& opers = assembly.generators.back().operators;
               opers.emplace_back();
               opers.back().name = read_string(line+20, 3);
               opers.back().transform = matrix;
@@ -510,11 +524,16 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
           st.info["_cell.Z_PDB"] = z;
       }
     } else if (is_record_type(line, "MTRIXn")) {
-      if (read_matrix(matrix, line, len) == 3 && !matrix.is_identity()) {
+      if (read_matrix(matrix, line, len) == 3) {
         std::string id = read_string(line+7, 3);
-        bool given = len > 59 && line[59] == '1';
-        st.ncs.push_back({id, given, matrix});
-        matrix.set_identity();
+        if (matrix.is_identity()) {
+          // store only ID that will be used when writing to file
+          st.info["_struct_ncs_oper.id"] = id;
+        } else {
+          bool given = len > 59 && line[59] == '1';
+          st.ncs.push_back({id, given, matrix});
+          matrix.set_identity();
+        }
       }
     } else if (is_record_type(line, "MODEL")) {
       if (model && chain)
@@ -529,10 +548,16 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
       model = nullptr;
       chain = nullptr;
 
-    } else if (is_record_type(line, "TER")) { // finishes polymer chains
+    } else if (is_record_type3(line, "TER")) { // finishes polymer chains
       // we don't expect more than one TER record in one chain
       if (!chain || after_ter)
         continue;
+      if (options.split_chain_on_ter) {
+        chain = nullptr;
+        // split_chain_on_ter is used for AMBER files that can have TER records
+        // in various places. So in such case TER doesn't imply entity_type.
+        continue;
+      }
       for (Residue& res : chain->residues)
         res.entity_type = EntityType::Polymer;
       after_ter = true;
@@ -586,7 +611,7 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
                is_record_type(line, "CISPEP")) {
       conn_records.emplace_back(line);
 
-    } else if (is_record_type(line, "END")) {  // NUL == ' ' & ~0x20
+    } else if (is_record_type3(line, "END")) {
       break;
     } else if (is_record_type(line, "data")) {
       if (line[4] == '_' && model && model->chains.empty())
@@ -612,32 +637,35 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
 
 }  // namespace pdb_impl
 
-inline Structure read_pdb_file(const std::string& path, int linelen=0) {
+inline Structure read_pdb_file(const std::string& path,
+                               PdbReadOptions options=PdbReadOptions()) {
   auto f = file_open(path.c_str(), "rb");
-  return pdb_impl::read_pdb_from_line_input(FileStream{f.get()}, path, linelen);
+  return pdb_impl::read_pdb_from_line_input(FileStream{f.get()}, path, options);
 }
 
 inline Structure read_pdb_from_memory(const char* data, size_t size,
-                                      const std::string& name, int linelen=0) {
+                                      const std::string& name,
+                                      PdbReadOptions options=PdbReadOptions()) {
   return pdb_impl::read_pdb_from_line_input(MemoryStream{data, data + size},
-                                            name, linelen);
+                                            name, options);
 }
 
 inline Structure read_pdb_string(const std::string& str,
-                                 const std::string& name) {
-  return read_pdb_from_memory(str.c_str(), str.length(), name);
+                                 const std::string& name,
+                                 PdbReadOptions options=PdbReadOptions()) {
+  return read_pdb_from_memory(str.c_str(), str.length(), name, options);
 }
 
 // A function for transparent reading of stdin and/or gzipped files.
 template<typename T>
-inline Structure read_pdb(T&& input, int linelen=0) {
+inline Structure read_pdb(T&& input, PdbReadOptions options=PdbReadOptions()) {
   if (input.is_stdin())
     return pdb_impl::read_pdb_from_line_input(FileStream{stdin},
-                                              "stdin", linelen);
+                                              "stdin", options);
   if (input.is_compressed())
     return pdb_impl::read_pdb_from_line_input(input.get_uncompressing_stream(),
-                                              input.path(), linelen);
-  return read_pdb_file(input.path(), linelen);
+                                              input.path(), options);
+  return read_pdb_file(input.path(), options);
 }
 
 } // namespace gemmi
