@@ -79,6 +79,48 @@ namespace gemmi {
 
 namespace impl {
 
+struct ItemSpan {
+  ItemSpan(std::vector<cif::Item>& items, const std::string& cat)
+      : items_(items), begin_(0), end_(items.size()) {
+    cif::assert_tag(cat);
+    while (begin_ != items.size() && !has_prefix(items[begin_], cat))
+      ++begin_;
+    if (begin_ != end_)
+      while (end_ - 1 != begin_ && !has_prefix(items[end_-1], cat))
+        --end_;
+  }
+
+  // cf. Block::set_pair()
+  void set_pair(const std::string& tag, const std::string& value) {
+    using namespace gemmi::cif;
+    assert_tag(tag);
+    for (size_t i = begin_; i != end_; ++i) {
+      Item& item = items_[i];
+      if (item.type == ItemType::Pair && item.pair[0] == tag) {
+        item.pair[1] = value;
+        return;
+      }
+      if (item.type == ItemType::Loop && item.loop.find_tag(tag) != -1) {
+        item.set_value(Item(tag, value));
+        return;
+      }
+    }
+    items_.emplace(items_.begin() + end_, tag, value);
+    ++end_;
+  }
+private:
+  std::vector<cif::Item>& items_;
+  size_t begin_, end_;
+
+  static bool has_prefix(const cif::Item& item, const std::string& cat) {
+    if (item.type == cif::ItemType::Pair)
+      return starts_with(item.pair[0], cat);
+    if (item.type == cif::ItemType::Loop)
+      return !item.loop.tags.empty() && starts_with(item.loop.tags[0], cat);
+    return false;
+  }
+};
+
 inline std::string pdbx_icode(const SeqId& seqid) {
   return std::string(1, seqid.has_icode() ? seqid.icode : '?');
 }
@@ -344,7 +386,7 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
       continue;
     const Atom* at1 = res1->find_atom(con.partner1.atom_name, con.partner1.altloc);
     const Atom* at2 = res2->find_atom(con.partner2.atom_name, con.partner2.altloc);
-    
+
     std::string im_pdb_symbol = "?", im_dist_str = "?";
     if (at1 && at2) {
       SymImage im = st.cell.find_nearest_image(at1->pos,
@@ -386,13 +428,13 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
       type_loop.add_row({connection_type_to_string((Connection::Type)i)});
 }
 
-void write_cell_parameters(const UnitCell& cell, cif::Block& block) {
-  block.set_pair("_cell.length_a",    to_str(cell.a));
-  block.set_pair("_cell.length_b",    to_str(cell.b));
-  block.set_pair("_cell.length_c",    to_str(cell.c));
-  block.set_pair("_cell.angle_alpha", to_str(cell.alpha));
-  block.set_pair("_cell.angle_beta",  to_str(cell.beta));
-  block.set_pair("_cell.angle_gamma", to_str(cell.gamma));
+void write_cell_parameters(const UnitCell& cell, ItemSpan& span) {
+  span.set_pair("_cell.length_a",    to_str(cell.a));
+  span.set_pair("_cell.length_b",    to_str(cell.b));
+  span.set_pair("_cell.length_c",    to_str(cell.c));
+  span.set_pair("_cell.angle_alpha", to_str(cell.alpha));
+  span.set_pair("_cell.angle_beta",  to_str(cell.beta));
+  span.set_pair("_cell.angle_gamma", to_str(cell.gamma));
 }
 
 void write_ncs_oper(const Structure& st, cif::Block& block) {
@@ -441,25 +483,28 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
     auto initial_date =
            st.info.find("_pdbx_database_status.recvd_initial_deposition_date");
     if (initial_date != st.info.end() && !initial_date->second.empty()) {
-      block.set_pair("_pdbx_database_status.entry_id", id);
-      block.set_pair(initial_date->first, initial_date->second);
+      impl::ItemSpan span(block.items, "_pdbx_database_status.");
+      span.set_pair("_pdbx_database_status.entry_id", id);
+      span.set_pair(initial_date->first, initial_date->second);
     }
   }
 
   if (groups.cell) {
-    block.set_pair("_cell.entry_id", id);
-    impl::write_cell_parameters(st.cell, block);
+    impl::ItemSpan cell_span(block.items, "_cell.");
+    cell_span.set_pair("_cell.entry_id", id);
+    impl::write_cell_parameters(st.cell, cell_span);
     auto z_pdb = st.info.find("_cell.Z_PDB");
     if (z_pdb != st.info.end())
-      block.set_pair(z_pdb->first, z_pdb->second);
+      cell_span.set_pair(z_pdb->first, z_pdb->second);
   }
 
   if (groups.symmetry) {
-    block.set_pair("_symmetry.entry_id", id);
-    block.set_pair("_symmetry.space_group_name_H-M",
+    impl::ItemSpan span(block.items, "_symmetry.");
+    span.set_pair("_symmetry.entry_id", id);
+    span.set_pair("_symmetry.space_group_name_H-M",
                    cif::quote(st.spacegroup_hm));
     if (const SpaceGroup* sg = st.find_spacegroup())
-      block.set_pair("_symmetry.Int_Tables_number", to_string(sg->number));
+      span.set_pair("_symmetry.Int_Tables_number", to_string(sg->number));
   }
 
   if (groups.entity) {
@@ -509,17 +554,19 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
               label_end = span.auth_seq_id_to_label(dbref.seq_end);
             } catch (const std::runtime_error&) {}
           }
-          seq_loop.add_row({std::to_string(++counter2),
-                            std::to_string(counter),
-                            subs.at(subchain),  // pdbx_strand_id
-                            std::to_string(*label_begin),
-                            std::to_string(*label_end),
-                            std::to_string(*dbref.db_begin.num),
-                            std::to_string(*dbref.db_end.num),
-                            dbref.seq_begin.num.str(),
-                            impl::pdbx_icode(dbref.seq_begin),
-                            dbref.seq_end.num.str(),
-                            impl::pdbx_icode(dbref.seq_end)});
+          auto strand_id = subs.find(subchain);
+          if (strand_id != subs.end())
+            seq_loop.add_row({std::to_string(++counter2),
+                              std::to_string(counter),
+                              strand_id->second,  // pdbx_strand_id
+                              std::to_string(*label_begin),
+                              std::to_string(*label_end),
+                              std::to_string(*dbref.db_begin.num),
+                              std::to_string(*dbref.db_end.num),
+                              dbref.seq_begin.num.str(),
+                              impl::pdbx_icode(dbref.seq_begin),
+                              dbref.seq_end.num.str(),
+                              impl::pdbx_icode(dbref.seq_end)});
         }
       }
   }
@@ -795,17 +842,19 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
   if (groups.title_keywords) {
     auto title = st.info.find("_struct.title");
     if (title != st.info.end()) {
-      block.set_pair("_struct.entry_id", id);
-      block.set_pair(title->first, cif::quote(title->second));
+      impl::ItemSpan span(block.items, "_struct.");
+      span.set_pair("_struct.entry_id", id);
+      span.set_pair(title->first, cif::quote(title->second));
     }
     auto pdbx_keywords = st.info.find("_struct_keywords.pdbx_keywords");
     auto keywords = st.info.find("_struct_keywords.text");
+    impl::ItemSpan span(block.items, "_struct_keywords.");
     if (pdbx_keywords != st.info.end() || keywords != st.info.end())
-      block.set_pair("_struct_keywords.entry_id", id);
+      span.set_pair("_struct_keywords.entry_id", id);
     if (pdbx_keywords != st.info.end())
-      block.set_pair(pdbx_keywords->first, cif::quote(pdbx_keywords->second));
+      span.set_pair(pdbx_keywords->first, cif::quote(pdbx_keywords->second));
     if (keywords != st.info.end())
-      block.set_pair(keywords->first, cif::quote(keywords->second));
+      span.set_pair(keywords->first, cif::quote(keywords->second));
   }
 
   if (groups.ncs)
@@ -825,14 +874,15 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
 
   if (groups.origx) { // _database_PDB_matrix (ORIGX)
     if (st.has_origx && !st.origx.is_identity()) {
-      block.set_pair("_database_PDB_matrix.entry_id", id);
+      impl::ItemSpan span(block.items, "_database_PDB_matrix.");
+      span.set_pair("_database_PDB_matrix.entry_id", id);
       std::string prefix = "_database_PDB_matrix.origx";
       for (int i = 0; i < 3; ++i) {
         std::string s = "[" + to_string(i+1) + "]";
-        block.set_pair(prefix + s + "[1]", to_str(st.origx.mat[i][0]));
-        block.set_pair(prefix + s + "[2]", to_str(st.origx.mat[i][1]));
-        block.set_pair(prefix + s + "[3]", to_str(st.origx.mat[i][2]));
-        block.set_pair(prefix + "_vector" + s, to_str(st.origx.vec.at(i)));
+        span.set_pair(prefix + s + "[1]", to_str(st.origx.mat[i][0]));
+        span.set_pair(prefix + s + "[2]", to_str(st.origx.mat[i][1]));
+        span.set_pair(prefix + s + "[3]", to_str(st.origx.mat[i][2]));
+        span.set_pair(prefix + "_vector" + s, to_str(st.origx.vec.at(i)));
       }
     }
   }
@@ -965,8 +1015,9 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
 
   // _pdbx_struct_assembly* and _struct_biol are REMARK 300/350 in PDB
   if (groups.struct_biol && !st.meta.remark_300_detail.empty()) {
-    block.set_pair("_struct_biol.id", "1");
-    block.set_pair("_struct_biol.details", cif::quote(st.meta.remark_300_detail));
+    impl::ItemSpan span(block.items, "_struct_biol.");
+    span.set_pair("_struct_biol.id", "1");
+    span.set_pair("_struct_biol.details", cif::quote(st.meta.remark_300_detail));
   }
   if (groups.assembly && !st.assemblies.empty())
     impl::write_assemblies(st, block);
@@ -992,15 +1043,16 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
 
   // _atom_sites (SCALE)
   if (groups.scale && (st.has_origx || st.cell.explicit_matrices)) {
-    block.set_pair("_atom_sites.entry_id", id);
+    impl::ItemSpan span(block.items, "_atom_sites.");
+    span.set_pair("_atom_sites.entry_id", id);
     std::string prefix = "_atom_sites.fract_transf_";
     for (int i = 0; i < 3; ++i) {
       std::string idx = "[" + std::to_string(i + 1) + "]";
       const auto& frac = st.cell.frac;
-      block.set_pair(prefix + "matrix" + idx + "[1]", to_str(frac.mat[i][0]));
-      block.set_pair(prefix + "matrix" + idx + "[2]", to_str(frac.mat[i][1]));
-      block.set_pair(prefix + "matrix" + idx + "[3]", to_str(frac.mat[i][2]));
-      block.set_pair(prefix + "vector" + idx, to_str(frac.vec.at(i)));
+      span.set_pair(prefix + "matrix" + idx + "[1]", to_str(frac.mat[i][0]));
+      span.set_pair(prefix + "matrix" + idx + "[2]", to_str(frac.mat[i][1]));
+      span.set_pair(prefix + "matrix" + idx + "[3]", to_str(frac.mat[i][2]));
+      span.set_pair(prefix + "vector" + idx, to_str(frac.vec.at(i)));
     }
   }
 
@@ -1111,7 +1163,8 @@ cif::Block make_mmcif_headers(const Structure& st) {
 }
 
 void add_minimal_mmcif_data(const Structure& st, cif::Block& block) {
-  impl::write_cell_parameters(st.cell, block);
+  impl::ItemSpan cell_span(block.items, "_cell.");
+  impl::write_cell_parameters(st.cell, cell_span);
   block.set_pair("_symmetry.space_group_name_H-M",
                  cif::quote(st.spacegroup_hm));
   impl::write_ncs_oper(st, block);
