@@ -6,6 +6,7 @@
 #define GEMMI_TO_PDB_HPP_
 
 #include "model.hpp"
+#include "polyheur.hpp"  // for are_connected
 #include <ostream>
 
 namespace gemmi {
@@ -17,6 +18,7 @@ struct PdbWriteOptions {
   bool cispep_records = true;
   bool ter_records = true;
   bool numbered_ter = true;
+  bool ter_ignores_type = false;
   bool use_linkr = false;
 };
 
@@ -80,7 +82,8 @@ bool use_hetatm(const Residue& res) {
     return true;
   if (res.het_flag == 'A')
     return false;
-  if (res.entity_type == EntityType::NonPolymer ||
+  if (res.entity_type == EntityType::Branched ||
+      res.entity_type == EntityType::NonPolymer ||
       res.entity_type == EntityType::Water)
     return true;
   return !find_tabulated_residue(res.name).is_standard();
@@ -149,7 +152,7 @@ inline void write_multiline(std::ostream& os, const char* record_name,
   for (int n = 2; n < 1000 && *end != '\0'; ++n) {
     start = end;
     end = find_last_break(start, lastcol-11);
-    int len = end - start;
+    int len = int(end - start);
     WRITEU("%-6s %3d %-69.*s\n", record_name, n, len, start);
   }
 }
@@ -163,14 +166,24 @@ inline void write_cryst1(const Structure& st, std::ostream& os) {
         st.get_info("_cell.Z_PDB").c_str());
 }
 
-inline void write_ncs(const Structure& st, std::ostream& os) {
+inline void write_ncs_op(const NcsOp& op, std::ostream& os) {
   char buf[88];
+  for (int i = 0; i < 3; ++i) {
+    WRITE("MTRIX%d %3.3s%10.6f%10.6f%10.6f %14.5f    %-21c\n", i+1,
+          op.id.c_str(), op.tr.mat[i][0], op.tr.mat[i][1], op.tr.mat[i][2],
+          op.tr.vec.at(i), op.given ? '1' : ' ');
+  }
+}
+
+inline void write_ncs(const Structure& st, std::ostream& os) {
+  if (st.ncs.empty())
+    return;
+  auto identity = st.info.find("_struct_ncs_oper.id");
+  if (identity != st.info.end() &&
+      !in_vector_f([&](const NcsOp& op) { return op.id == identity->second; }, st.ncs))
+    write_ncs_op(NcsOp{identity->second, true, {}}, os);
   for (const NcsOp& op : st.ncs)
-    for (int i = 0; i < 3; ++i) {
-      WRITE("MTRIX%d %3.3s%10.6f%10.6f%10.6f %14.5f    %-21c\n", i+1,
-            op.id.c_str(), op.tr.mat[i][0], op.tr.mat[i][1], op.tr.mat[i][2],
-            op.tr.vec.at(i), op.given ? '1' : ' ');
-    }
+    write_ncs_op(op, os);
 }
 
 inline void write_remarks(const Structure& st, std::ostream& os) {
@@ -240,7 +253,7 @@ inline void write_remarks(const Structure& st, std::ostream& os) {
           WRITELN("REMARK 350                    AND CHAINS: %s",
                   chains_str.substr(begin, end - begin).c_str());
         }
-        for (const Assembly::Oper& oper : gen.opers) {
+        for (const Assembly::Operator& oper : gen.operators) {
           ++oper_cnt;
           const Transform& tr = oper.transform;
           for (int i = 0; i < 3; ++i)
@@ -312,20 +325,23 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
             // Sometimes PDB files have explicit 0s (5M05); we ignore them.
             a.charge ? a.charge > 0 ? '0'+a.charge : '0'-a.charge : ' ',
             a.charge ? a.charge > 0 ? '+' : '-' : ' ');
-      if (a.u11 != 0.0f) {
+      if (a.aniso.nonzero()) {
         // re-using part of the buffer
         std::memcpy(buf, "ANISOU", 6);
         const double eps = 1e-6;
         gf_snprintf(buf+28, 43, "%7.0f%7.0f%7.0f%7.0f%7.0f%7.0f",
-                    a.u11*1e4 + eps, a.u22*1e4 + eps, a.u33*1e4 + eps,
-                    a.u12*1e4 + eps, a.u13*1e4 + eps, a.u23*1e4 + eps);
+                    a.aniso.u11*1e4 + eps, a.aniso.u22*1e4 + eps,
+                    a.aniso.u33*1e4 + eps, a.aniso.u12*1e4 + eps,
+                    a.aniso.u13*1e4 + eps, a.aniso.u23*1e4 + eps);
         buf[28+42] = ' ';
         os.write(buf, 81);
       }
     }
-    if (opt.ter_records && res.entity_type == EntityType::Polymer &&
-        (&res == &chain.residues.back() ||
-         (&res + 1)->entity_type != EntityType::Polymer)) {
+    if (opt.ter_records &&
+        (opt.ter_ignores_type ? &res == &chain.residues.back()
+                              : (res.entity_type == EntityType::Polymer &&
+                                (&res == &chain.residues.back() ||
+                                 (&res + 1)->entity_type != EntityType::Polymer)))) {
       if (opt.numbered_ter) {
         // re-using part of the buffer in the middle, e.g.:
         // TER    4153      LYS B 286
@@ -380,7 +396,7 @@ inline void write_header(const Structure& st, std::ostream& os,
     // "classification" in PDB == _struct_keywords.pdbx_keywords in mmCIF
     const std::string& keywords = st.get_info("_struct_keywords.pdbx_keywords");
     if (!pdb_date.empty() || !keywords.empty() || !entry_id.empty())
-      WRITEU("HEADER    %-40s%-9s   %-18s\n",
+      WRITEU("HEADER    %-40.40s%-9s   %-18.18s\n",
              keywords.c_str(), pdb_date.c_str(), entry_id.c_str());
   }
   write_multiline(os, "TITLE", st.get_info("_struct.title"), 80);
@@ -566,34 +582,44 @@ inline void write_header(const Structure& st, std::ostream& os,
     if (!st.models.empty() && opt.link_records) {
       for (const Connection& con : st.connections)
         if (con.type == Connection::Covale || con.type == Connection::MetalC ||
-            con.type == Connection::None) {
-          const_CRA cra1 = st.models[0].find_cra(con.partner1);
-          const_CRA cra2 = st.models[0].find_cra(con.partner2);
-          if (!cra1.atom || !cra2.atom)
+            con.type == Connection::Unknown) {
+          const Residue* res1 = st.models[0].find_residue(con.partner1.chain_name, con.partner1.res_id);
+          const Residue* res2 = st.models[0].find_residue(con.partner2.chain_name, con.partner2.res_id);
+          if (!res1 || !res2)
             continue;
-          SymImage im = st.cell.find_nearest_image(cra1.atom->pos,
-                                                   cra2.atom->pos, con.asu);
+          const Atom* at1 = res1->find_atom(con.partner1.atom_name, con.partner1.altloc);
+          const Atom* at2 = res2->find_atom(con.partner2.atom_name, con.partner2.altloc);
+          
+          std::string im_pdb_symbol = "", im_dist_str = "";
+          bool im_same_asu = true;
+          if (at1 && at2) {
+            SymImage im = st.cell.find_nearest_image(at1->pos,
+                                                     at2->pos, con.asu);
+            im_pdb_symbol = im.pdb_symbol(false);
+            im_dist_str = to_str_prec<2>(im.dist());
+            im_same_asu = im.same_asu();
+          }
           // Pdb spec: "sym1 and sym2 are right justified and are given as
           // blank when the identity operator (and no cell translation) is
           // to be applied to the atom." But all files from wwPDB have
           // 1555 not blank, so here we also write 1555,
           // except for LINKR (Refmac variant of LINK).
           gf_snprintf(buf, 82, "LINK        %-4s%c%3s%2s%5s   "
-                "            %-4s%c%3s%2s%5s  %6s %6s %5.2f  \n",
-                padded_atom_name(*cra1.atom).c_str(),
-                cra1.atom->altloc ? std::toupper(cra1.atom->altloc) : ' ',
-                cra1.residue->name.c_str(),
-                cra1.chain->name.c_str(),
-                write_seq_id(buf8, cra1.residue->seqid),
-                padded_atom_name(*cra2.atom).c_str(),
-                cra2.atom->altloc ? std::toupper(cra2.atom->altloc) : ' ',
-                cra2.residue->name.c_str(),
-                cra2.chain->name.c_str(),
-                write_seq_id(buf8a, cra2.residue->seqid),
-                "1555", im.pdb_symbol(false).c_str(), im.dist());
+                "            %-4s%c%3s%2s%5s  %6s %6s %5s  \n",
+                at1 ? padded_atom_name(*at1).c_str() : "",
+                at1 && at1->altloc ? std::toupper(at1->altloc) : ' ',
+                res1->name.c_str(),
+                con.partner1.chain_name.c_str(),
+                write_seq_id(buf8, res1->seqid),
+                at2 ? padded_atom_name(*at2).c_str() : "",
+                at2 && at2->altloc ? std::toupper(at2->altloc) : ' ',
+                res2->name.c_str(),
+                con.partner2.chain_name.c_str(),
+                write_seq_id(buf8a, res2->seqid),
+                "1555", im_pdb_symbol.c_str(), im_dist_str.c_str());
           if (opt.use_linkr && !con.link_id.empty()) {
             buf[4] = 'R';  // LINK -> LINKR
-            if (im.same_asu())
+            if (im_same_asu)
               std::memset(buf+58, ' ', 14); // erase symmetry
             // overwrite distance with link_id
             gf_snprintf(buf+72, 82-72, "%-8s\n", con.link_id.c_str());
@@ -609,8 +635,9 @@ inline void write_header(const Structure& st, std::ostream& os,
       for (const Model& model : st.models)
         for (const Chain& chain : model.chains)
           for (const Residue& res : chain.residues)
-            if (res.is_cis)
-              if (const Residue* next = chain.next_bonded_aa(res)) {
+            if (res.is_cis) {
+              const Residue* next = chain.next_residue(res);
+              if (next && are_connected(res, *next, PolymerType::PeptideL)) {
                 if (++counter == 10000)
                   counter = 0;
                 WRITE("CISPEP%4d %3s%2s %5s   %3s%2s %5s %9s %12.2f %20s\n",
@@ -623,6 +650,7 @@ inline void write_header(const Structure& st, std::ostream& os,
                       deg(calculate_omega(res, *next)),
                       "");
               }
+            }
     }
   }
 
@@ -643,15 +671,24 @@ inline void write_header(const Structure& st, std::ostream& os,
   write_ncs(st, os);
 }
 
+inline void check_if_structure_can_be_written_as_pdb(const Structure& st) {
+  for (const gemmi::Model& model : st.models)
+    for (const gemmi::Chain& chain : model.chains)
+      if (chain.name.size() > 2)
+        gemmi::fail("chain name too long for the PDB format: " + chain.name);
+}
+
 } // namespace impl
 
 std::string make_pdb_headers(const Structure& st) {
+  impl::check_if_structure_can_be_written_as_pdb(st);
   std::ostringstream os;
   impl::write_header(st, os, PdbWriteOptions());
   return os.str();
 }
 
 void write_pdb(const Structure& st, std::ostream& os, PdbWriteOptions opt) {
+  impl::check_if_structure_can_be_written_as_pdb(st);
   impl::write_header(st, os, opt);
   impl::write_atoms(st, os, opt);
   char buf[88];
@@ -660,6 +697,7 @@ void write_pdb(const Structure& st, std::ostream& os, PdbWriteOptions opt) {
 
 void write_minimal_pdb(const Structure& st, std::ostream& os,
                        PdbWriteOptions opt) {
+  impl::check_if_structure_can_be_written_as_pdb(st);
   impl::write_cryst1(st, os);
   impl::write_ncs(st, os);
   impl::write_atoms(st, os, opt);
