@@ -2,7 +2,7 @@
 
 #include "gemmi/unitcell.hpp"
 #include "gemmi/refln.hpp"
-#include "gemmi/fourier.hpp"
+#include "gemmi/fourier.hpp"  // for get_size_for_hkl, get_f_phi_on_grid, ...
 #include "gemmi/tostr.hpp"
 #include "gemmi/fprime.hpp"
 #include "gemmi/reciproc.hpp"  // for count_reflections, make_miller_vector
@@ -107,6 +107,18 @@ void add_hkl(py::module& m) {
        py::arg("exact_size")=std::array<int,3>{{0,0,0}},
        py::arg("sample_rate")=0.,
        py::arg("order")=AxisOrder::XYZ)
+    .def("get_float", &make_asu_data<float, ReflnBlock>,
+         py::arg("col"), py::arg("as_is")=false)
+    .def("get_f_phi", [](const ReflnBlock& self, const std::string& f_col,
+                                                 const std::string& phi_col,
+                                                 bool as_is) {
+        return make_asu_data<std::complex<float>, 2>(self, {f_col, phi_col}, as_is);
+    }, py::arg("f"), py::arg("phi"), py::arg("as_is")=false)
+    .def("get_value_sigma", [](const ReflnBlock& self, const std::string& f_col,
+                                                       const std::string& sigma_col,
+                                                       bool as_is) {
+        return make_asu_data<ValueSigma<float>, 2>(self, {f_col, sigma_col}, as_is);
+    }, py::arg("f"), py::arg("sigma"), py::arg("as_is")=false)
     .def("is_unmerged", &ReflnBlock::is_unmerged)
     .def("use_unmerged", &ReflnBlock::use_unmerged)
     .def("__bool__", [](const ReflnBlock& self) { return self.ok(); })
