@@ -98,6 +98,7 @@ void add_mol(py::module& m) {
     .def_readwrite("assemblies", &Structure::assemblies)
     .def_readwrite("info", &Structure::info)
     .def_readwrite("raw_remarks", &Structure::raw_remarks)
+    .def("find_spacegroup", &Structure::find_spacegroup)
     .def("get_entity",
          (Entity* (Structure::*)(const std::string&)) &Structure::get_entity,
          py::arg("subchain"), py::return_value_policy::reference_internal)
@@ -212,6 +213,7 @@ void add_mol(py::module& m) {
     .def("__delitem__", remove_child<Model>, py::arg("index"))
     .def("__delitem__", remove_children<Model>)
     .def("count_atom_sites", &count_atom_sites<Model>)
+    .def("count_hydrogen_sites", &count_hydrogen_sites<Model>)
     .def("count_occupancies", &count_occupancies<Model>)
     .def("calculate_mass", &calculate_mass<Model>)
     .def("calculate_center_of_mass", [](const Model& self) {
@@ -221,7 +223,11 @@ void add_mol(py::module& m) {
     .def("__repr__", [](const Model& self) {
         return tostr("<gemmi.Model ", self.name, " with ",
                      self.chains.size(), " chain(s)>");
-    });
+    })
+    .def("transform", [](Model& self, const Transform& tr) {
+        for (CRA& cra : self.all())
+          transform_atom(*cra.atom, tr);
+    }, py::arg("tr"));
 
   py::class_<UniqProxy<Residue>>(m, "FirstConformerRes")
     .def("__iter__", [](UniqProxy<Residue>& self) {
@@ -268,12 +274,16 @@ void add_mol(py::module& m) {
          py::arg("residue"), py::arg("pos")=-1,
          py::return_value_policy::reference_internal)
     .def("subchains", (std::vector<ResidueSpan> (Chain::*)()) &Chain::subchains)
-    .def("whole", (ResidueSpan (Chain::*)()) &Chain::whole)
-    .def("get_polymer", (ResidueSpan (Chain::*)()) &Chain::get_polymer)
-    .def("get_ligands", (ResidueSpan (Chain::*)()) &Chain::get_ligands)
-    .def("get_waters", (ResidueSpan (Chain::*)()) &Chain::get_waters)
-    .def("get_subchain",
-         (ResidueSpan (Chain::*)(const std::string&)) &Chain::get_subchain)
+    .def("whole", (ResidueSpan (Chain::*)()) &Chain::whole,
+         py::keep_alive<0, 1>())
+    .def("get_polymer", (ResidueSpan (Chain::*)()) &Chain::get_polymer,
+         py::keep_alive<0, 1>())
+    .def("get_ligands", (ResidueSpan (Chain::*)()) &Chain::get_ligands,
+         py::keep_alive<0, 1>())
+    .def("get_waters", (ResidueSpan (Chain::*)()) &Chain::get_waters,
+         py::keep_alive<0, 1>())
+    .def("get_subchain", (ResidueSpan (Chain::*)(const std::string&)) &Chain::get_subchain,
+         py::keep_alive<0, 1>())
     .def("has_subchains_assigned", &has_subchains_assigned)
     .def("previous_residue", &Chain::previous_residue,
          py::return_value_policy::reference_internal)
@@ -388,16 +398,14 @@ void add_mol(py::module& m) {
          py::arg("name"), py::return_value_policy::reference_internal)
     .def("__delitem__", remove_child<Residue>, py::arg("index"))
     .def("__delitem__", remove_children<Residue>)
-    .def("find_atom", [](Residue& self, const std::string& name, char altloc) {
-           return self.find_atom(name, altloc);
-         },
-         //(Atom* (Residue::*)(const std::string&, char, El)) &Residue::find_atom,
-         py::arg("name"), py::arg("altloc"), /*py::arg("el")=El::X,*/
-         py::return_value_policy::reference_internal)
+    .def("find_atom", [](Residue& self, const std::string& name, char altloc, Element el) {
+           return self.find_atom(name, altloc, el);
+         }, py::arg("name"), py::arg("altloc"), py::arg("el")=Element(El::X),
+            py::return_value_policy::reference_internal)
     .def("remove_atom",
-         [](Residue& self, const std::string& name, char altloc/*, El el*/) {
-           self.atoms.erase(self.find_atom_iter(name, altloc/*, el*/));
-    }, py::arg("name"), py::arg("altloc") /*, py::arg("el")=El::X*/)
+         [](Residue& self, const std::string& name, char altloc, Element el) {
+           self.atoms.erase(self.find_atom_iter(name, altloc, el));
+    }, py::arg("name"), py::arg("altloc"), py::arg("el")=Element(El::X))
     .def("add_atom", add_child<Residue, Atom>,
          py::arg("atom"), py::arg("pos")=-1,
          py::return_value_policy::reference_internal)
@@ -405,7 +413,10 @@ void add_mol(py::module& m) {
          (UniqProxy<Atom> (Residue::*)()) &Residue::first_conformer,
          py::keep_alive<0, 1>())
     .def("sole_atom", &Residue::sole_atom)
+    .def("get_ca", &Residue::get_ca, py::return_value_policy::reference_internal)
+    .def("get_p", &Residue::get_p, py::return_value_policy::reference_internal)
     .def("is_water", &Residue::is_water)
+    .def("remove_hydrogens", &remove_hydrogens<Residue>)
     .def("trim_to_alanine", (bool (*)(Residue&)) &trim_to_alanine)
     .def("clone", [](const Residue& self) { return new Residue(self); })
     .def("__repr__", [](const Residue& self) {
