@@ -1,6 +1,7 @@
 // Copyright 2017 Global Phasing Ltd.
 
 #include "gemmi/unitcell.hpp"
+#include "gemmi/tostr.hpp"  // tostr
 
 #include <cstdio>  // for snprintf
 #include <array>
@@ -21,12 +22,16 @@ static std::string triple(double x, double y, double z) {
   return std::string(buf);
 }
 
-void mat33_from_list(Mat33& self, std::array<std::array<double,3>,3>& m) {
+static void mat33_from_list(Mat33& self, std::array<std::array<double,3>,3>& m) {
   for (int i = 0; i < 3; ++i)
     for (int j = 0; j < 3; ++j)
       self.a[i][j] = m[i][j];
 }
 
+static py::tuple make_parameters_tuple(const UnitCell& u) {
+  return py::make_tuple(u.a, u.b, u.c, u.alpha, u.beta, u.gamma);
+}
+
 template<typename T> void add_smat33(py::module& m, const char* name) {
   using M = SMat33<T>;
   py::class_<M>(m, name)
@@ -61,7 +66,10 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
     }, py::arg().noconvert())
     .def("transformed_by", &M::template transformed_by<double>)
     .def("calculate_eigenvalues", &M::calculate_eigenvalues)
-    ;
+    .def("__repr__", [name](const M& m) {
+        return tostr("<gemmi.", name, '(', m.u11, ", ", m.u22, ", ", m.u33, ", ",
+                     m.u12, ", ", m.u13, ", ", m.u23, + ")>");
+    });
 }
 
 template<typename T> void add_box(py::module& m, const char* name) {
@@ -221,9 +229,7 @@ void add_unitcell(py::module& m) {
     .def_readonly("gamma", &UnitCell::gamma)
     .def_readonly("volume", &UnitCell::volume)
     .def_readonly("images", &UnitCell::images)
-    .def_property_readonly("parameters", [](const UnitCell& self) {
-        return py::make_tuple(self.a, self.b, self.c, self.alpha, self.beta, self.gamma);
-    })
+    .def_property_readonly("parameters", &make_parameters_tuple)
     .def_property_readonly("fractionalization_matrix",
                            [](const UnitCell& self) { return self.frac.mat; })
     .def_property_readonly("orthogonalization_matrix",
@@ -236,6 +242,10 @@ void add_unitcell(py::module& m) {
     .def("volume_per_image", &UnitCell::volume_per_image)
     .def("find_nearest_image", &UnitCell::find_nearest_image,
          py::arg("ref"), py::arg("pos"), py::arg("asu")=Asu::Any)
+    .def("find_nearest_pbc_image", &UnitCell::find_nearest_pbc_image,
+         py::arg("ref"), py::arg("pos"), py::arg("image_idx"))
+    .def("find_nearest_pbc_position", &UnitCell::find_nearest_pbc_position,
+         py::arg("ref"), py::arg("pos"), py::arg("image_idx"), py::arg("inverse")=false)
     .def("is_special_position",
          (int (UnitCell::*)(const Position&, double) const)
            &UnitCell::is_special_position,
@@ -260,5 +270,12 @@ void add_unitcell(py::module& m) {
     .def("__repr__", [](const UnitCell& self) {
         return "<gemmi.UnitCell(" + triple(self.a, self.b, self.c)
              + ", " + triple(self.alpha, self.beta, self.gamma) + ")>";
-    });
+    })
+    .def(py::pickle(
+         &make_parameters_tuple,
+         [](const py::tuple p) {
+            return UnitCell (p[0].cast<double>(), p[1].cast<double>(), p[2].cast<double>(),
+                             p[3].cast<double>(), p[4].cast<double>(), p[5].cast<double>());
+         }
+    ));
 }
